/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/bincimapd-capability.cc
 *  
 *  Description:
 *    Implementation of the CAPABILITY command.
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>

#include "depot.h"
#include "io/io.h"
#include "operators.h"
#include "parsers/imap/recursivedescent/recursivedescent.h"
#include "session.h"

using namespace ::std;
using namespace Binc;

//----------------------------------------------------------------------
CapabilityOperator::CapabilityOperator(void)
{
}

//----------------------------------------------------------------------
CapabilityOperator::~CapabilityOperator(void)
{
}

//----------------------------------------------------------------------
const string CapabilityOperator::getName(void) const
{
  return "CAPABILITY";
}

//----------------------------------------------------------------------
int CapabilityOperator::getState(void) const
{
  return Session::NONAUTHENTICATED 
    | Session::AUTHENTICATED
    | Session::SELECTED;
}

//----------------------------------------------------------------------
bool CapabilityOperator::parse(Command &c_in) const
{
  try {
    if (!expectThisString("CAPABILITY"))
      return false;
    if (!expectCRLF())
      throw SyntaxException("expected CRLF");
  } catch (InputException &e) {
    e.addReason("in CapabilityOperator::parse");
    throw e;
  }

  c_in.setName("CAPABILITY");
  return true;
}

//----------------------------------------------------------------------
void CapabilityOperator::process(Depot &depot,
				 Command &command) const
{
  IO &com = IOFactory::getInstance().get(1);
  Session &session = Session::getInstance();

  com << "*" << " CAPABILITY IMAP4rev1";

  session.globalconfig.setSection("Authentication");

  if (session.getState() == Session::NONAUTHENTICATED) {
    if (session.command.ssl &&
	(session.globalconfig["disable starttls"] != "yes"))
      com << " STARTTLS";

    const bool allowplain 
      = (session.globalconfig["allow plain auth in non ssl"] == "yes");
    
    if (session.command.ssl || allowplain)
      com << " AUTH=LOGIN AUTH=PLAIN";
    else
      com << " LOGINDISABLED";
  }

  vector<string>::const_iterator i = capabilities.begin();
  while (i != capabilities.end()) {
    com << " " << *i;
    ++i;
  }
  com << endl;
}

//----------------------------------------------------------------------
void CapabilityOperator::addCapability(const string &cap)
{
  capabilities.push_back(cap);
}
