/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/operator-close.cc
 *  
 *  Description:
 *    Implementation of the CLOSE command.
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#include <string>

#include "depot.h"
#include "io/io.h"
#include "mailbox/mailbox.h"
#include "operators.h"
#include "parsers/imap/recursivedescent/recursivedescent.h"
#include "session.h"

using namespace ::std;
using namespace Binc;

//----------------------------------------------------------------------
CloseOperator::CloseOperator(void)
{
}

//----------------------------------------------------------------------
CloseOperator::~CloseOperator(void)
{
}

//------------------------------------------------------------------------
const string CloseOperator::getName(void) const
{
  return "CLOSE";
}

//------------------------------------------------------------------------
int CloseOperator::getState(void) const
{
  return Session::SELECTED;
}

//------------------------------------------------------------------------
void CloseOperator::process(Depot &depot,
			    Command &command) const
{
  IO &logger = IOFactory::getInstance().get(2);

  Mailbox *mailbox = depot.getSelected();
  if (mailbox == NULL)
    throw CriticalException("close attempted with no mailbox selected");

  mailbox->imapExpunge();
  mailbox->imapClose();

  Session &session = Session::getInstance();
  session.setState(Session::AUTHENTICATED);

  logger.setLogPrefix(session.getUserID() + "@" + session.getIP() + ":");
}

//----------------------------------------------------------------------
bool CloseOperator::parse(Command &c_in) const
{
  try {
    if (!expectThisString("CLOSE"))
      return false;
    if (!expectCRLF())
      throw SyntaxException("expected CRLF");
  } catch (InputException &e) {
    e.addReason("in CloseOperator::parse");
    throw e;
  }

  c_in.setName("CLOSE");
  return true;
}
