/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/operator-expunge.cc
 *  
 *  Description:
 *    Implementation of the EXPUNGE command
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#include <string>
    
#include "depot.h"
#include "io/io.h"
#include "mailbox/mailbox.h"
#include "operators.h"
#include "parsers/imap/imapparser.h"
#include "parsers/imap/recursivedescent/recursivedescent.h"
#include "pendingupdates.h"
#include "session.h"

using namespace ::std;
using namespace Binc;

//----------------------------------------------------------------------
ExpungeOperator::ExpungeOperator(void)
{
}

//----------------------------------------------------------------------
ExpungeOperator::~ExpungeOperator(void)
{
}

//----------------------------------------------------------------------
const string ExpungeOperator::getName(void) const
{
  return "EXPUNGE";
}

//----------------------------------------------------------------------
int ExpungeOperator::getState(void) const
{
  return Session::SELECTED;
}

//----------------------------------------------------------------------
void ExpungeOperator::process(Depot &depot,
			      Command &command) const
{
  Mailbox *mailbox = depot.getSelected();
  if (mailbox == NULL)
    throw CriticalException("expunge when no mailbox is selected");

  mailbox->imapExpunge();

  pendingUpdates(mailbox, PendingUpdates::EXPUNGE
		 | PendingUpdates::EXISTS 
		 | PendingUpdates::RECENT
		 | PendingUpdates::FLAGS, true);
}

//----------------------------------------------------------------------
bool ExpungeOperator::parse(Command &c_in) const
{
  try {
    if (!expectThisString("EXPUNGE"))
      return false;
    if (!expectCRLF())
      throw SyntaxException("expected CRLF");
  } catch (InputException &e) {
    e.addReason("in ExpungeOperator::parse");
    throw e;
  }

  c_in.setName("EXPUNGE");
  return true;
}
