/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/parsers/mime/mime-parseonlyheader.cc
 *  
 *  Description:
 *    Implementation of main mime parser components
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "mime.h"
#include "mime-utils.h"
#include "../../util/convert.h"
#include "../../io/io.h"
#include <string>
#include <vector>
#include <map>
#include <exception>
#include <iostream>

#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include <errno.h>

using namespace ::std;

//------------------------------------------------------------------------
void Binc::MimeDocument::parseOnlyHeader(FILE *fp) const
{
  if (allIsParsed || headerIsParsed)
    return;
  
  headerIsParsed = true;

  while (CRLFbuffer.size() != 0)
    CRLFbuffer.pop();
  CRLFlastchar = '\0';
  CRLFbuffersize = 0;

  headerstartoffset = 0;
  headerlength = 0;
  bodystartoffset = 0;
  bodylength = 0;
  //  size = -1;
  messagerfc822 = false;
  multipart = false;

  nlines = 0;
  nbodylines = 0;

  int crlfnr = 0;
  int bytenr = 0;
  MimePart::parseOnlyHeader(fp, "", bytenr, crlfnr);
}

//------------------------------------------------------------------------
int Binc::MimePart::parseOnlyHeader(FILE *fp, const string &toboundary, int &bytenr, int &crlfnr) const
{
  string name;
  string content;
  char cqueue[4];
  memset(cqueue, 0, sizeof(cqueue));

  headerstartoffset = bytenr;
  int headerstartoffsetcrlf = crlfnr;

  bool quit = false;
  int c = '\0';

  while (!quit) {
    // read name
    while (1) {
      if ((c = fgetcCRLF(fp)) == EOF) {
	quit = true;
	break;
      }

      ++crlfnr;
      if (c == '\n') ++nlines;
      if (c != '\r') ++bytenr;
      if (c == ':') break;
      if (c == '\n') {
	--bytenr;
	for (int i = name.length() - 1; i >= 0; --i) {
	  fungetcCRLF(name[i]);
	  --crlfnr;
	  if (name[i] != '\r') --bytenr;
	}
	quit = true;
	name = "";
	break;
      }

      if (c == EOF) {
	quit = true;
	break;
      }

      name += c;

      if (name.length() == 2 && name.substr(0, 2) == "\r\n") {
	name = "";
	quit = true;
	break;
      }
    }

    if (name.length() == 1 && name[0] == '\r') {
      name = "";
      break;
    }

    if (quit) break;

    while (!quit) {
      c = fgetcCRLF(fp);
      if (c == EOF) {
	quit = true;
	break;
      }

      ++crlfnr;
      if (c == '\n') ++nlines;
      if (c != '\r') ++bytenr;

      for (int i = 0; i < 3; ++i)
	cqueue[i] = cqueue[i + 1];
      cqueue[3] = c;

      if (strncmp(cqueue, "\r\n\r\n", 4) == 0) {
	quit = true;
	break;
      }

      if (cqueue[2] == '\n') {

	// guess the mime rfc says what can not appear on the beginning
	// of a line.
	if (!isspace(cqueue[3])) {
	  if (content.length() > 2)
	    content.resize(content.length() - 2);

	  trim(content);
	  h.add(name, content);

	  name = c;
	  content = "";
	  break;
	}
      }

      content += c;
    }
  }

  if (name != "") {
    if (content.length() > 2)
      content.resize(content.length() - 2);
    h.add(name, content);
  }

  headerlength = crlfnr - headerstartoffsetcrlf;
  if (headerlength < 0) headerlength = 0;

  return 1;
}
