/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/session.h
 *  
 *  Description:
 *    <--->
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifndef session_h_included
#define session_h_included
#include "parsers/storage/storage.h"
#include "parsers/args/arg-parser.h"
#include <string>
#include <vector>

namespace Binc {
  
  //--------------------------------------------------------------------
  class Session {
    //--
    int state;
    std::string userid;
    std::string ip;
    char **argv;
    int argc;

    int logfacility;

    int readbytes;
    int writebytes;
    int statements;
    int bodies;

    //--
    Session(void);

    ArgParser::Args args;
  public:

    //--
    std::map<std::string, std::string> attrs;

    //-- Command line variables
    std::vector<std::string> restArgs;
    char ** unparsedArgs;
    struct {
      bool help;
      bool version;
      bool ssl;
      std::string configfile;
    } command;

    //--
    enum {NONAUTHENTICATED = 0x01,
	  AUTHENTICATED = 0x02,
	  SELECTED = 0x04,
	  LOGOUT = 0x00 };

    //--
    Storage globalconfig;
    Storage localconfig;
    std::vector<std::string> subscribed;
    
    //--
    const int getState(void) const;
    void setState(int n);

    //--
    void exportToEnv(void) throw (BincException &);
    void importFromEnv(void) throw (BincException &);

    //--
    void parseCommandLine(int argc, char * argv[]);

    //--
    int getWriteBytes(void) const;
    int getReadBytes(void) const;

    //--
    void addWriteBytes(int);
    void addReadBytes(int);

    //--
    int getBodies(void) const;
    int getStatements(void) const;

    //--
    void addBody(void);
    void addStatement(void);

    //--
    void setLogFacility(int facility);
    int getLogFacility(void) const;

    //--
    const std::string &getIP(void) const;
    void setIP(const std::string &ip);

    //--
    const std::string &getUserID() const;
    void setUserID(const std::string &s);

    //--
    const std::string &operator [] (const std::string &) const;

    //--
    void initConfig(void);

    //--
    void add(const std::string &, const std::string &);

    //--
    static Session &getInstance(void);
  };
}

#endif
